@@ -1,11 +1,22 @@
-"""Authentication library, implemented by users of the API."""
+"""Authentication library, implemented by users of the API.
+
+This library is a simple `aiohttp` that handles authentication when talking
+to the API. Users are expected to provide their own implementation that provides
+credentials obtained using the standard Google authentication approaches
+described at https://developers.google.com/nest/device-access/api/authorization
+
+An implementation of `AbstractAuth` implements `async_get_access_token`
+to provide authentication credentials to the SDM library. The implementation is
+responsible for managing the lifecycle of the token (any persistence needed,
+or refresh to deal with expiration, etc).
+"""
 
 from __future__ import annotations
 
 import logging
 from abc import ABC, abstractmethod
 from asyncio import TimeoutError
-from typing import Any, List, Mapping, Optional
+from typing import Any, Mapping
 
 import aiohttp
 from aiohttp.client_exceptions import ClientError, ClientResponseError
@@ -16,6 +27,8 @@ from .exceptions import ApiException, AuthException
 
 _LOGGER = logging.getLogger(__name__)
 
+__all__ = ["AbstractAuth"]
+
 HTTP_UNAUTHORIZED = 401
 AUTHORIZATION_HEADER = "Authorization"
 ERROR = "error"
@@ -27,7 +40,7 @@ class AbstractAuth(ABC):
     """Abstract class to make authenticated requests."""
 
     def __init__(self, websession: aiohttp.ClientSession, host: str):
-        """Initialize the auth."""
+        """Initialize the AbstractAuth."""
         self._websession = websession
         self._host = host
 
@@ -41,7 +54,10 @@ class AbstractAuth(ABC):
         return OAuthCredentials(token=token)
 
     async def request(
-        self, method: str, url: str, **kwargs: Optional[Mapping[str, Any]]
+        self,
+        method: str,
+        url: str,
+        **kwargs: Mapping[str, Any] | None,
     ) -> aiohttp.ClientResponse:
         """Make a request."""
         headers = kwargs.get("headers")
@@ -74,7 +90,7 @@ class AbstractAuth(ABC):
         method: str,
         url: str,
         headers: dict[str, str],
-        **kwargs: Optional[Mapping[str, Any]],
+        **kwargs: Mapping[str, Any] | None,
     ) -> aiohttp.ClientResponse:
         return await self._websession.request(method, url, **kwargs, headers=headers)
 
@@ -132,8 +148,8 @@ class AbstractAuth(ABC):
         return resp
 
     @staticmethod
-    async def _error_detail(resp: aiohttp.ClientResponse) -> List[str]:
-        """Resturns an error message string from the APi response."""
+    async def _error_detail(resp: aiohttp.ClientResponse) -> list[str]:
+        """Returns an error message string from the APi response."""
         if resp.status < 400:
             return []
         try:
