@@ -2,12 +2,12 @@
 
 from __future__ import annotations
 
-from typing import Awaitable, Callable, Dict, Optional
+from typing import Awaitable, Callable, Dict
 
-from .device import Device
+from .device import Device, ParentRelation
 from .event import EventMessage, RelationUpdate
 from .event_media import CachePolicy
-from .structure import InfoTrait, RoomInfoTrait, Structure
+from .structure import Structure
 
 
 class DeviceManager:
@@ -71,12 +71,12 @@ class DeviceManager:
                 device = self._devices[device_id]
                 await device.async_handle_event(event_message)
 
-    def _structure_name(self, relation_subject: str) -> Optional[str]:
+    def _structure_name(self, relation_subject: str) -> str:
         if relation_subject in self._structures:
             structure = self._structures[relation_subject]
-            for trait_name in [InfoTrait.NAME, RoomInfoTrait.NAME]:
-                if trait_name in structure.traits:
-                    return structure.traits[trait_name].custom_name
+            for trait in [structure.info, structure.room_info]:
+                if trait and trait.custom_name:
+                    return trait.custom_name
         return "Unknown"
 
     def _handle_device_relation(self, relation: RelationUpdate) -> None:
@@ -86,12 +86,16 @@ class DeviceManager:
         device = self._devices[relation.object]
         if relation.type == "DELETED":
             # Delete device from room/structure
-            if relation.subject in device.parent_relations:
-                del device.parent_relations[relation.subject]
+            device.delete_relation(relation.subject)
 
         if relation.type == "UPDATED" or relation.type == "CREATED":
             # Device moved to a room
             assert relation.subject
-            device.parent_relations[relation.subject] = self._structure_name(
-                relation.subject
+            device.create_relation(
+                ParentRelation.from_dict(
+                    {
+                        "parent": relation.subject,
+                        "displayName": self._structure_name(relation.subject),
+                    }
+                )
             )
