@@ -1,4 +1,12 @@
-"""Libraries related to providing a device level interface for event related media."""
+"""Libraries related to providing a device level interface for event related media.
+
+An `EventMediaManager` is associated with a single device and manages the
+state for events and the lifecycle of media for those events. The manager is
+invoked by the subscriber when new events arrive and it handles any fetching
+related to the media, as well as transcoding video clips of needed. The
+`CachePolicy` settings determine lifecycle options such as how many events
+to keep around in the underlying store.
+"""
 
 from __future__ import annotations
 
@@ -10,31 +18,44 @@ import time
 from abc import ABC, abstractmethod
 from collections import OrderedDict
 from collections.abc import Iterable
-from typing import Any, Awaitable, Callable, Dict
+from dataclasses import dataclass, field
+from typing import Any, Awaitable, Callable, cast
+
+from mashumaro import DataClassDictMixin
+from mashumaro.types import SerializationStrategy
+from mashumaro.config import BaseConfig
 
 from .camera_traits import (
     CameraClipPreviewTrait,
     CameraEventImageTrait,
     EventImageContentType,
-    EventImageGenerator,
 )
 from .diagnostics import EVENT_MEDIA_DIAGNOSTICS as DIAGNOSTICS
 from .diagnostics import Diagnostics
 from .event import (
     CameraClipPreviewEvent,
-    CameraMotionEvent,
-    CameraPersonEvent,
-    CameraSoundEvent,
-    DoorbellChimeEvent,
     EventImageType,
     EventMessage,
     EventToken,
     ImageEventBase,
     session_event_image_type,
+    CameraPersonEvent,
+    CameraMotionEvent,
+    CameraSoundEvent,
+    DoorbellChimeEvent,
 )
 from .exceptions import GoogleNestException, TranscodeException
 from .transcoder import Transcoder
 
+__all__ = [
+    "EventMediaManager",
+    "ImageSession",
+    "ClipPreviewSession",
+    "Media",
+    "EventMediaStore",
+    "CachePolicy",
+]
+
 _LOGGER = logging.getLogger(__name__)
 
 # Should be large enough for processing, but not too large to be a size issue
@@ -54,176 +75,48 @@ VISIBLE_EVENTS = [
 EXPIRE_CACHE_BATCH_SIZE = 0.05
 
 
-class CachePolicy:
-    """Policy for how many local objects to cache in memory."""
-
-    def __init__(self, event_cache_size: int = DEFAULT_CACHE_SIZE, fetch: bool = False):
-        self._event_cache_size = event_cache_size
-        self._fetch = fetch
-        self._store: EventMediaStore = InMemoryEventMediaStore()
-        self._transcoder: Transcoder | None = None
-
-    @property
-    def event_cache_size(self) -> int:
-        """Number of events to keep in memory per device."""
-        return self._event_cache_size
-
-    @event_cache_size.setter
-    def event_cache_size(self, value: int) -> None:
-        """Set the number of events to keep in memory per device."""
-        self._event_cache_size = value
-
-    @property
-    def event_cache_expire_count(self) -> int:
-        """Number of events to keep in memory per device."""
-        return max(int(self._event_cache_size * EXPIRE_CACHE_BATCH_SIZE), 1)
-
-    @property
-    def fetch(self) -> bool:
-        """Return true if event media should be pre-fetched."""
-        return self._fetch
-
-    @fetch.setter
-    def fetch(self, value: bool) -> None:
-        """Update the value for whether event media should be pre-fetched."""
-        self._fetch = value
-
-    @property
-    def store(self) -> EventMediaStore:
-        """Return the EventMediaStore object."""
-        return self._store
-
-    @store.setter
-    def store(self, value: EventMediaStore) -> None:
-        """Update the EventMediaStore."""
-        self._store = value
-
-    @property
-    def transcoder(self) -> Transcoder | None:
-        """Return the transcoder."""
-        return self._transcoder
-
-    @transcoder.setter
-    def transcoder(self, value: Transcoder) -> None:
-        """Update the Transcoder."""
-        self._transcoder = value
-
-
+@dataclass
 class Media:
     """Represents media related to an event."""
 
-    def __init__(
-        self, contents: bytes, event_image_type: EventImageContentType
-    ) -> None:
-        """Initialize Media."""
-        self._contents = contents
-        self._event_image_type = event_image_type
+    contents: bytes
+    """Media content."""
 
-    @property
-    def contents(self) -> bytes:
-        """Media content."""
-        return self._contents
-
-    @property
-    def event_image_type(self) -> EventImageContentType:
-        """Content event image type of the media."""
-        return self._event_image_type
+    event_image_type: EventImageContentType
+    """Content event image type of the media."""
 
     @property
     def content_type(self) -> str:
         """Content type of the media."""
-        return self._event_image_type.content_type
+        return self.event_image_type.content_type
 
 
-class ImageSession(ABC):
-    """An object that holds events that happened within a time range."""
+@dataclass
+class ImageSession:
+    """An object that holds an image based event."""
 
-    def __init__(
-        self,
-        event_token: str,
-        event_timestamp: datetime.datetime,
-        event_type: str,
-    ) -> None:
-        self._event_token = event_token
-        self._event_timestamp = event_timestamp
-        self._event_type = event_type
+    event_token: str
+    """A token that can be used to fetch the media for the event."""
 
-    @property
-    def event_token(self) -> str:
-        return self._event_token
+    timestamp: datetime.datetime
+    """Timestamp when the event happened."""
 
-    @property
-    def timestamp(self) -> datetime.datetime:
-        """Return timestamp that the event ocurred."""
-        return self._event_timestamp
-
-    @property
-    def event_type(self) -> str:
-        return self._event_type
+    event_type: str
+    """A label for the type of event."""
 
 
-class ClipPreviewSession(ABC):
-    """An object that holds events that happened within a time range."""
+@dataclass
+class ClipPreviewSession:
+    """An object that holds a clip based event."""
 
-    def __init__(
-        self,
-        event_token: str,
-        event_timestamp: datetime.datetime,
-        event_types: list[str],
-    ) -> None:
-        self._event_token = event_token
-        self._event_timestamp = event_timestamp
-        self._event_types = event_types
+    event_token: str
+    """A token that can be used to fetch the media for the event."""
 
-    @property
-    def event_token(self) -> str:
-        return self._event_token
+    timestamp: datetime.datetime
+    """Timestamp when the event happened."""
 
-    @property
-    def timestamp(self) -> datetime.datetime:
-        """Return timestamp that the event ocurred."""
-        return self._event_timestamp
-
-    @property
-    def event_types(self) -> list[str]:
-        return self._event_types
-
-
-class EventMedia:
-    """Represents an event and its associated media."""
-
-    def __init__(
-        self,
-        event_session_id: str,
-        event_id: str,
-        event_type: str,
-        event_timestamp: datetime.datetime,
-        media: Media,
-    ) -> None:
-        self._event_session_id = event_session_id
-        self._event_id = event_id
-        self._event_type = event_type
-        self._event_timestamp = event_timestamp
-        self._media = media
-
-    @property
-    def event_session_id(self) -> str:
-        """Return the event session id."""
-        return self._event_session_id
-
-    @property
-    def event_type(self) -> str:
-        """Return the event type."""
-        return self._event_type
-
-    @property
-    def event_timestamp(self) -> datetime.datetime:
-        """Return timestamp that the event ocurred."""
-        return self._event_timestamp
-
-    @property
-    def media(self) -> Media:
-        return self._media
+    event_types: list[str]
+    """A label for the type of event."""
 
 
 class EventMediaStore(ABC):
@@ -287,7 +180,10 @@ class InMemoryEventMediaStore(EventMediaStore):
 
     def get_image_media_key(self, device_id: str, event: ImageEventBase) -> str:
         """Return the media key to use for the device and event."""
-        return f"{device_id}_{event.timestamp}_{event.event_session_id}_{event.event_id}.jpg"
+        return (
+            f"{device_id}_{event.timestamp}_{event.event_session_id}_"
+            f"{event.event_id}.jpg"
+        )
 
     def get_clip_preview_media_key(self, device_id: str, event: ImageEventBase) -> str:
         """Return the media key to use for the device and event."""
@@ -316,172 +212,121 @@ class InMemoryEventMediaStore(EventMediaStore):
                 del self._media[media_key]
 
 
-class EventMediaModelItem:
-    """Structure used to persist the event in EventMediaStore."""
+@dataclass
+class CachePolicy:
+    """Policy for how many local objects to cache in memory."""
 
-    def __init__(
-        self,
-        event_session_id: str,
-        events: dict[str, ImageEventBase],
-        media_key: str | None,
-        event_media_keys: dict[str, str],
-        thumbnail_media_key: str | None,
-        pending_event_keys: set[str] | None = None,
-    ) -> None:
-        """Initialize EventMediaModelItem."""
-        self._event_session_id = event_session_id
-        self._events = events
-        self._media_key = media_key
-        self._event_media_keys = event_media_keys if event_media_keys else {}
-        self._thumbnail_media_key = thumbnail_media_key
-        self._pending_event_keys = pending_event_keys if pending_event_keys else set({})
+    event_cache_size: int = DEFAULT_CACHE_SIZE
+    """Number of events to keep in memory per device."""
 
-    @staticmethod
-    def from_dict(data: dict[str, Any]) -> EventMediaModelItem:
-        """Read from serialized dictionary."""
+    fetch: bool = False
+    """Determine if event media should be pre-fetched."""
+
+    store: EventMediaStore = field(default_factory=InMemoryEventMediaStore)
+    """The EventMediaStore object for storing media content."""
+
+    transcoder: Transcoder | None = None
+    """The transcoder for encoding media."""
+
+    @property
+    def event_cache_expire_count(self) -> int:
+        """Number of events to keep in memory per device."""
+        return max(int(self.event_cache_size * EXPIRE_CACHE_BATCH_SIZE), 1)
+
+
+class ImageEventSerializationStrategy(SerializationStrategy):
+    """Serialization strategy for ImageEventBase."""
+
+    def serialize(self, value: dict[str, ImageEventBase]) -> dict[str, Any]:
+        """Serialize ImageEventBase."""
+        return dict((k, v.as_dict()) for k, v in value.items())
+
+    def deserialize(self, value: dict[str, Any]) -> dict[str, ImageEventBase]:
+        """Deserialize ImageEventBase."""
         events: dict[str, ImageEventBase] = {}
-        input_events = data.get("events", {})
-        for (event_type, event_data) in input_events.items():
-            if not (event := ImageEventBase.from_dict(event_data)):
-                continue
-            events[event_type] = event
+        for event_type, event_data in value.items():
+            # Propagate timestamps to child nodes
+            if (timestamp := event_data.get("timestamp")) and (
+                data := event_data.get("event_data")
+            ):
+                data["timestamp"] = timestamp
+            if event := ImageEventBase.parse_event_dict(event_data):
+                events[event_type] = event
+
         # Link events to other events in the session
         event_image_type = session_event_image_type(events.values())
         for event in events.values():
-            event.session_events = list(events.values())
             event.event_image_type = event_image_type
-        return EventMediaModelItem(
-            data["event_session_id"],
-            events,
-            data.get("media_key"),
-            data.get("event_media_keys", {}),
-            data.get("thumbnail_media_key"),
-            set(data.get("pending_event_keys", [])),
-        )
+        return events
 
-    @property
-    def event_session_id(self) -> str:
-        return self._event_session_id
+
+@dataclass
+class EventMediaModelItem(DataClassDictMixin):
+    """Structure used to persist the event in EventMediaStore."""
+
+    event_session_id: str
+    events: dict[str, ImageEventBase] = field(default_factory=dict)
+    media_key: str | None = field(default=None)
+    event_media_keys: dict[str, str] = field(default_factory=dict)
+    thumbnail_media_key: str | None = field(default=None)
+    pending_event_keys: set[str] = field(default_factory=set)
 
     @property
     def visible_event(self) -> ImageEventBase | None:
         """Get the primary visible event for this item."""
         for event_type in VISIBLE_EVENTS:
-            if event := self._events.get(event_type):
+            if event := self.events.get(event_type):
                 return event
         return None
 
-    @property
-    def events(self) -> dict[str, ImageEventBase]:
-        """Return all associated events with this record."""
-        return self._events
-
     def merge_events(self, new_events: dict[str, ImageEventBase]) -> None:
         """Merge new incoming events with the existing set."""
-        new_keys = new_events.keys() - self._events.keys()
-        self._events.update(new_events)
-        self._pending_event_keys |= new_keys
-
-    @property
-    def pending_event_keys(self) -> set[str]:
-        """Return the set of events that have not yet been notified."""
-        return self._pending_event_keys
+        new_keys = new_events.keys() - self.events.keys()
+        self.events.update(new_events)
+        self.pending_event_keys |= new_keys
 
     @property
     def pending_events(self) -> dict[str, ImageEventBase]:
         """Return all associated events with this record."""
         return {
             key: value
-            for key, value in self._events.items()
+            for key, value in self.events.items()
             if key in self.pending_event_keys
         }
 
     def notified(self, event_keys: Iterable[str]) -> None:
         """Mark the specified events as notified."""
-        self._pending_event_keys = self._pending_event_keys - set(event_keys)
-
-    @property
-    def media_key(self) -> str | None:
-        return self._media_key
-
-    @media_key.setter
-    def media_key(self, value: str) -> None:
-        """Update the media_key."""
-        self._media_key = value
-
-    @property
-    def event_media_keys(self) -> dict[str, str]:
-        return self._event_media_keys
+        self.pending_event_keys = self.pending_event_keys - set(event_keys)
 
     def media_key_for_token(self, token: EventToken) -> str | None:
         """Return media key for the specified event token."""
         if token.event_id:
-            if token.event_id in self._event_media_keys:
-                return self._event_media_keys[token.event_id]
+            if token.event_id in self.event_media_keys:
+                return self.event_media_keys[token.event_id]
             # Fallback to legacy single event per session
-        return self._media_key
+        return self.media_key
 
     @property
     def any_media_key(self) -> str | None:
         """Return any media item for compatibility with legacy APIs."""
-        if self._media_key:
-            return self._media_key
-        if self._event_media_keys.values():
-            return next(iter(self._event_media_keys.values()))
+        if self.media_key:
+            return self.media_key
+        if self.event_media_keys.values():
+            return next(iter(self.event_media_keys.values()))
         return None
 
     @property
     def all_media_keys(self) -> list[str]:
         """Return all media items for purging media keys."""
-        keys: list[str | None] = [self.media_key, self.thumbnail_media_key]
-        keys.extend(self.event_media_keys.values())
+        keys = [self.media_key, self.thumbnail_media_key] + list(
+            self.event_media_keys.values()
+        )
         return [key for key in keys if key is not None]
 
-    @property
-    def thumbnail_media_key(self) -> str | None:
-        return self._thumbnail_media_key
-
-    @thumbnail_media_key.setter
-    def thumbnail_media_key(self, value: str) -> None:
-        """Update the thumbnail_media_key."""
-        self._thumbnail_media_key = value
-
-    def get_media(self, content: bytes) -> Media:
-        assert self.visible_event
-        return Media(content, self.visible_event.event_image_type)
-
-    def get_event_media(self, content: bytes) -> EventMedia:
-        assert self.visible_event
-        return EventMedia(
-            self.visible_event.event_session_id,
-            self.visible_event.event_id,
-            self.visible_event.event_type,
-            self.visible_event.timestamp,
-            self.get_media(content),
-        )
-
-    def as_dict(self) -> dict[str, Any]:
-        """Return a EventMediaModelItem as a serializable dict."""
-        result: dict[str, Any] = {
-            "event_session_id": self._event_session_id,
-            "events": dict((k, v.as_dict()) for k, v in self._events.items()),
-            "event_media_keys": self._event_media_keys,
-            "pending_event_keys": list(self._pending_event_keys),
+    class Config(BaseConfig):
+        serialization_strategy = {
+            dict[str, ImageEventBase]: ImageEventSerializationStrategy(),
         }
-        if self._media_key:
-            result["media_key"] = self._media_key
-        if self._thumbnail_media_key:
-            result["thumbnail_media_key"] = self._thumbnail_media_key
-        return result
-
-    def __repr__(self) -> str:
-        return (
-            "<EventMediaModelItem events="
-            + str(self._events)
-            + " media_key="
-            + str(self._media_key)
-            + ">"
-        )
 
 
 class EventMediaManager:
@@ -490,18 +335,20 @@ class EventMediaManager:
     def __init__(
         self,
         device_id: str,
-        traits: Dict[str, Any],
-        event_trait_map: Dict[str, EventImageGenerator],
-        support_fetch: bool,
+        traits: dict[str, Any],
+        event_traits: set[str],
         diagnostics: Diagnostics,
     ) -> None:
         """Initialize DeviceEventMediaManager."""
         self._device_id = device_id
         self._traits = traits
-        self._event_trait_map = event_trait_map
+        self._event_traits = event_traits
         self._cache_policy = CachePolicy()
         self._callback: Callable[[EventMessage], Awaitable[None]] | None = None
-        self._support_fetch = support_fetch
+        self._support_fetch = (
+            CameraClipPreviewTrait.NAME in traits
+            or CameraEventImageTrait.NAME in traits
+        )
         self._diagnostics = diagnostics
         self._lock: asyncio.Lock | None = None
 
@@ -522,7 +369,11 @@ class EventMediaManager:
         if store_data:
             device_data = store_data.get(self._device_id, [])
             for item_data in device_data:
-                item = EventMediaModelItem.from_dict(item_data)
+                try:
+                    item = EventMediaModelItem.from_dict(item_data)
+                except Exception as err:
+                    _LOGGER.debug("Failed to parse event item: %s", str(err))
+                    raise err
                 event_data[item.event_session_id] = item
         return event_data
 
@@ -531,14 +382,14 @@ class EventMediaManager:
         # Event order is preserved so popping from the oldest entry works
         device_data: list[dict[str, Any]] = []
         for item in event_data.values():
-            device_data.append(item.as_dict())
+            device_data.append(item.to_dict())
 
         # Read data from the store and update information for this device
         store_data = await self._cache_policy.store.async_load()
         if not store_data:
             store_data = {}
         store_data[self._device_id] = device_data
-        await self._cache_policy._store.async_save(store_data)
+        await self._cache_policy.store.async_save(store_data)
 
     async def _async_load_item(
         self, event_session_id: str
@@ -580,49 +431,6 @@ class EventMediaManager:
                     await self._cache_policy.store.async_remove_media(media_key)
             await self._async_update(event_data)
 
-    async def get_media(
-        self, event_session_id: str, width: int | None = None, height: int | None = None
-    ) -> EventMedia | None:
-        """Get media for the specified event.
-
-        Note that the height and width hints are best effort and may not be
-        honored (e.g. if image is already cached).
-        """
-        self._diagnostics.increment("get_media")
-        if not (item := await self._async_load_item(event_session_id)):
-            return None
-        event = item.visible_event
-        if not event:
-            _LOGGER.debug(
-                "Skipping fetch; No visible event; event_session_id=%s",
-                event_session_id,
-            )
-            return None
-        media_key = item.any_media_key
-        if media_key:
-            contents = await self._cache_policy._store.async_load_media(media_key)
-            if contents:
-                return item.get_event_media(contents)
-
-        if event.is_expired:
-            _LOGGER.debug(
-                "Skipping fetch; Event expired; event_session_id=%s",
-                event_session_id,
-            )
-            return None
-        _LOGGER.debug("Fetching media for event_session_id=%s", event_session_id)
-        if not (generator := self._event_trait_map.get(event.event_type)):
-            return None
-        event_image = await generator.generate_event_image(event)
-        if not event_image:
-            return None
-        contents = await event_image.contents(width=width, height=height)
-        media_key = self._cache_policy._store.get_media_key(self._device_id, event)
-        await self._cache_policy.store.async_save_media(media_key, contents)
-        item.media_key = media_key
-        await self._async_update_item(item)
-        return item.get_event_media(contents)
-
     async def _fetch_media(self, item: EventMediaModelItem) -> None:
         """Fetch media from the server in response to a pubsub event."""
         store = self._cache_policy.store
@@ -631,7 +439,12 @@ class EventMediaManager:
             if (
                 item.media_key
                 or not item.visible_event
-                or not (clip_event := item.events.get(CameraClipPreviewEvent.NAME))
+                or not (
+                    clip_event := cast(
+                        CameraClipPreviewEvent,
+                        item.events.get(CameraClipPreviewEvent.NAME),
+                    )
+                )
                 or clip_event.is_expired
             ):
                 self._diagnostics.increment("fetch_clip.skip")
@@ -639,7 +452,9 @@ class EventMediaManager:
             clip_preview_trait: CameraClipPreviewTrait = self._traits[
                 CameraClipPreviewTrait.NAME
             ]
-            event_image = await clip_preview_trait.generate_event_image(clip_event)
+            event_image = await clip_preview_trait.generate_event_image(
+                clip_event.preview_url
+            )
             if event_image:
                 content = await event_image.contents()
                 # Caller will persist the media key assignment
@@ -687,7 +502,7 @@ class EventMediaManager:
             self._diagnostics.increment("get_media.no_media")
             _LOGGER.debug("No persisted media for event id %s", token)
             return None
-        contents = await self._cache_policy._store.async_load_media(media_key)
+        contents = await self._cache_policy.store.async_load_media(media_key)
         if not contents:
             self._diagnostics.increment("get_media.empty")
             _LOGGER.debug(
@@ -717,7 +532,7 @@ class EventMediaManager:
 
         if item.thumbnail_media_key:
             # Load cached thumbnail
-            contents = await self._cache_policy._store.async_load_media(
+            contents = await self._cache_policy.store.async_load_media(
                 item.thumbnail_media_key
             )
             if contents:
@@ -735,7 +550,7 @@ class EventMediaManager:
             return None
 
         thumbnail_media_key = (
-            self._cache_policy._store.get_clip_preview_thumbnail_media_key(
+            self._cache_policy.store.get_clip_preview_thumbnail_media_key(
                 self._device_id, item.visible_event
             )
         )
@@ -753,7 +568,7 @@ class EventMediaManager:
             _LOGGER.debug("Failure to transcode clip thumbnail: %s", str(err))
             return None
 
-        contents = await self._cache_policy._store.async_load_media(thumbnail_media_key)
+        contents = await self._cache_policy.store.async_load_media(thumbnail_media_key)
         if not contents:
             self._diagnostics.increment("get_clip.load_error")
             _LOGGER.debug(
@@ -767,19 +582,6 @@ class EventMediaManager:
         self._diagnostics.increment("get_clip.success")
         return Media(contents, EventImageType.IMAGE_PREVIEW)
 
-    async def async_events(self) -> list[ImageEventBase]:
-        """Return revent events."""
-        self._diagnostics.increment("load_events")
-        result = await self._visible_items()
-
-        def _get_event(x: EventMediaModelItem) -> ImageEventBase:
-            assert x.visible_event
-            return x.visible_event
-
-        event_result = list(map(_get_event, result))
-        event_result.sort(key=lambda x: x.timestamp, reverse=True)
-        return event_result
-
     async def async_image_sessions(self) -> list[ImageSession]:
         """Return revent events."""
         self._diagnostics.increment("load_image_sessions")
@@ -804,7 +606,7 @@ class EventMediaManager:
         return event_result
 
     async def async_clip_preview_sessions(self) -> list[ClipPreviewSession]:
-        """Return revent events."""
+        """Return revent events for a device that supports clips."""
         self._diagnostics.increment("load_clip_previews")
 
         def _event_visible(x: ImageEventBase) -> bool:
@@ -832,20 +634,8 @@ class EventMediaManager:
         valid_clips.sort(key=lambda x: x.timestamp, reverse=True)
         return valid_clips
 
-    async def _visible_items(self) -> list[EventMediaModelItem]:
-        """Return items in the modle that are visible events for serving."""
-
-        def _filter(x: EventMediaModelItem) -> bool:
-            """Return events already fetched or that could be fetched."""
-            return x.visible_event is not None and (
-                x.media_key is not None or not x.visible_event.is_expired
-            )
-
-        event_data = await self._async_load()
-        return list(filter(_filter, event_data.values()))
-
     async def _items_with_media(self) -> list[EventMediaModelItem]:
-        """Return items in the modle that have media for serving."""
+        """Return items in the model that have media for serving."""
 
         def _filter(x: EventMediaModelItem) -> bool:
             """Return events already fetched or that could be fetched."""
@@ -865,9 +655,9 @@ class EventMediaManager:
     async def async_handle_events(self, event_message: EventMessage) -> None:
         """Handle the EventMessage."""
         self._diagnostics.increment("event")
-        event_sessions: dict[
-            str, dict[str, ImageEventBase]
-        ] | None = event_message.event_sessions
+        event_sessions: dict[str, dict[str, ImageEventBase]] | None = (
+            event_message.event_sessions
+        )
         if not event_sessions:
             return
         _LOGGER.debug("Event Update %s", event_sessions.keys())
@@ -882,12 +672,11 @@ class EventMediaManager:
                     self._diagnostics.elapsed(event_name, recv_latency_ms)
                 else:
                     self._diagnostics.elapsed(f"{event_name}_expired", recv_latency_ms)
-                if not (trait := self._event_trait_map.get(event_name)):
+                if event_name not in self._event_traits:
                     self._diagnostics.increment(f"event.unsupported.{event_name}")
                     _LOGGER.debug("Unsupported event trait: %s", event_name)
                     continue
                 supported = True
-                trait.handle_event(event)
 
             # Skip any entirely unsupported events
             if not supported:
@@ -904,8 +693,8 @@ class EventMediaManager:
                 self._diagnostics.increment("event.new")
                 # A new event session
                 model_item = EventMediaModelItem(
-                    event_session_id,
-                    event_dict,
+                    event_session_id=event_session_id,
+                    events=event_dict,
                     media_key=None,
                     event_media_keys={},
                     thumbnail_media_key=None,
@@ -948,41 +737,10 @@ class EventMediaManager:
                 self._diagnostics.increment("event.notify")
                 await self._callback(event_message)
         else:
-            _LOGGER.debug(
-                "Message did not contain notifiable events",
-            )
-            # self._diagnostics.increment("event.notify_skip")
+            _LOGGER.debug("Message did not contain notifiable events")
 
         for model_item in model_items:
             model_item.notified(pending_events.keys())
             await self._async_update_item(model_item)
 
         await self._expire_cache()
-
-    @property
-    def active_event_trait(self) -> EventImageGenerator | None:
-        """Return trait with the most recently received active event."""
-        trait_to_return: EventImageGenerator | None = None
-        for trait in self._event_trait_map.values():
-            if not trait.active_event:
-                continue
-            if trait_to_return is None:
-                trait_to_return = trait
-            else:
-                event = trait.last_event
-                if not event or not trait_to_return.last_event:
-                    continue
-                if event.expires_at > trait_to_return.last_event.expires_at:
-                    trait_to_return = trait
-        return trait_to_return
-
-    async def get_active_event_media(
-        self, width: int | None = None, height: int | None = None
-    ) -> EventMedia | None:
-        """Return a snapshot image for a very recent event if any."""
-        if not (trait := self.active_event_trait):
-            return None
-        event: ImageEventBase | None = trait.last_event
-        if not event:
-            return None
-        return await self.get_media(event.event_session_id, width, height)
