@@ -1,4 +1,5 @@
 """Subscriber for the Smart Device Management event based API."""
+
 from __future__ import annotations
 
 import asyncio
@@ -9,11 +10,11 @@ import logging
 import re
 import time
 from abc import ABC, abstractmethod
-from typing import Awaitable, Callable, Optional
+from typing import Awaitable, Callable
 
 from aiohttp.client_exceptions import ClientError
 from google.api_core.exceptions import GoogleAPIError, NotFound, Unauthenticated
-from google.auth.exceptions import RefreshError
+from google.auth.exceptions import RefreshError, GoogleAuthError, TransportError
 from google.auth.transport.requests import Request
 from google.cloud import pubsub_v1
 from google.oauth2.credentials import Credentials
@@ -24,9 +25,20 @@ from .device_manager import DeviceManager
 from .diagnostics import SUBSCRIBER_DIAGNOSTICS as DIAGNOSTICS
 from .event import EventMessage
 from .event_media import CachePolicy
-from .exceptions import AuthException, ConfigurationException, SubscriberException
+from .exceptions import (
+    AuthException,
+    ConfigurationException,
+    SubscriberException,
+    ApiException,
+)
 from .google_nest_api import GoogleNestAPI
 
+__all__ = [
+    "GoogleNestSubscriber",
+    "ApiEnv",
+]
+
+
 _LOGGER = logging.getLogger(__name__)
 
 # Used to catch invalid subscriber id
@@ -45,6 +57,8 @@ WATCHDOG_RESET_THRESHOLD_SECONDS = 60
 
 DEFAULT_MESSAGE_RETENTION_SECONDS = 15 * 60  # 15 minutes
 
+MESSAGE_ACK_TIMEOUT_SECONDS = 30.0
+
 # Note: Users of non-prod instances will have to manually configure a topic
 TOPIC_FORMAT = "projects/sdm-prod/topics/enterprise-{project_id}"
 
@@ -92,7 +106,7 @@ def get_api_env(env: str | None) -> ApiEnv:
 
 
 def _validate_subscription_name(subscription_name: str) -> None:
-    """Validates that a subcription name is correct.
+    """Validates that a subscription name is correct.
 
     Raises ConfigurationException on failure.
     """
@@ -119,6 +133,26 @@ def _validate_topic_name(topic_name: str) -> None:
         )
 
 
+def refresh_creds(creds: Credentials) -> Credentials:
+    """Refresh credentials.
+
+    This is not part of the subscriber API, exposed only to facilitate testing.
+    """
+    try:
+        creds.refresh(Request())
+    except RefreshError as err:
+        raise AuthException(f"Authentication refresh failure: {err}") from err
+    except TransportError as err:
+        raise SubscriberException(
+            f"Connectivity error during authentication refresh: {err}"
+        ) from err
+    except GoogleAuthError as err:
+        raise SubscriberException(
+            f"Error during authentication refresh: {err}"
+        ) from err
+    return creds
+
+
 class AbstractSubscriberFactory(ABC):
     """Abstract class for creating a subscriber, to facilitate testing."""
 
@@ -167,14 +201,13 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
         """Creates a subscription name if it does not already exist."""
         _validate_subscription_name(subscription_name)
         _validate_topic_name(topic_name)
-        with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
-            await loop.run_in_executor(
-                executor,
-                self._create_subscription,
-                creds,
-                subscription_name,
-                topic_name,
-            )
+        await loop.run_in_executor(
+            None,
+            self._create_subscription,
+            creds,
+            subscription_name,
+            topic_name,
+        )
 
     def _create_subscription(
         self,
@@ -218,13 +251,12 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
         loop: asyncio.AbstractEventLoop,
     ) -> None:
         """Creates a subscription name if it does not already exist."""
-        with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
-            await loop.run_in_executor(
-                executor,
-                self._delete_subscription,
-                creds,
-                subscription_name,
-            )
+        await loop.run_in_executor(
+            None,
+            self._delete_subscription,
+            creds,
+            subscription_name,
+        )
 
     def _delete_subscription(
         self,
@@ -232,7 +264,7 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
         subscription_name: str,
     ) -> None:
         """Deletes a subscription."""
-        creds = self._refresh_creds(creds)
+        creds = refresh_creds(creds)
         subscriber = pubsub_v1.SubscriberClient(credentials=creds)
         _LOGGER.debug(f"Deleting subscription '{subscription_name}'")
         subscriber.delete_subscription(subscription=subscription_name)
@@ -249,29 +281,18 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
         """Create a new subscriber with a blocking to async bridge."""
 
         def callback_wrapper(message: pubsub_v1.subscriber.message.Message) -> None:
-            if loop.is_closed():
-                return
             future: concurrent.futures.Future = asyncio.run_coroutine_threadsafe(
                 async_callback(message), loop
             )
             future.result()
 
-        with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
-            return await loop.run_in_executor(
-                executor,
-                self._new_subscriber,
-                creds,
-                subscription_name,
-                callback_wrapper,
-            )
-
-    def _refresh_creds(self, creds: Credentials) -> Credentials:
-        """Refresh credentials."""
-        try:
-            creds.refresh(Request())
-        except RefreshError as err:
-            raise AuthException(f"Access token failure: {err}") from err
-        return creds
+        return await loop.run_in_executor(
+            None,
+            self._new_subscriber,
+            creds,
+            subscription_name,
+            callback_wrapper,
+        )
 
     def _new_subscriber(
         self,
@@ -280,7 +301,7 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
         callback_wrapper: Callable[[pubsub_v1.subscriber.message.Message], None],
     ) -> pubsub_v1.subscriber.futures.StreamingPullFuture:
         """Issue a command to verify subscriber creds are correct."""
-        creds = self._refresh_creds(creds)
+        creds = refresh_creds(creds)
         subscriber = pubsub_v1.SubscriberClient(credentials=creds)
         subscription = subscriber.get_subscription(subscription=subscription_name)
         if subscription.topic:
@@ -302,7 +323,7 @@ class GoogleNestSubscriber:
         project_id: str,
         subscriber_id: str,
         subscriber_factory: AbstractSubscriberFactory = DefaultSubscriberFactory(),
-        loop: Optional[asyncio.AbstractEventLoop] = None,
+        loop: asyncio.AbstractEventLoop | None = None,
         watchdog_check_interval_seconds: float = WATCHDOG_CHECK_INTERVAL_SECONDS,
         watchdog_restart_delay_min_seconds: float = WATCHDOG_RESTART_DELAY_MIN_SECONDS,
     ):
@@ -311,18 +332,18 @@ class GoogleNestSubscriber:
         self._subscriber_id = subscriber_id
         self._project_id = project_id
         self._api = GoogleNestAPI(auth, project_id)
-        self._loop = loop or asyncio.get_event_loop()
-        self._device_manager_task: Optional[asyncio.Task[DeviceManager]] = None
+        self._loop = loop or asyncio.get_running_loop()
+        self._device_manager_task: asyncio.Task[DeviceManager] | None = None
         self._subscriber_factory = subscriber_factory
-        self._subscriber_future: Optional[
-            pubsub_v1.subscriber.futures.StreamingPullFuture
-        ] = None
-        self._callback: Optional[Callable[[EventMessage], Awaitable[None]]] = None
+        self._subscriber_future: (
+            pubsub_v1.subscriber.futures.StreamingPullFuture | None
+        ) = None  # noqa: E501
+        self._callback: Callable[[EventMessage], Awaitable[None]] | None = None
         self._healthy = True
         self._watchdog_check_interval_seconds = watchdog_check_interval_seconds
         self._watchdog_restart_delay_min_seconds = watchdog_restart_delay_min_seconds
         self._watchdog_restart_delay_seconds = watchdog_restart_delay_min_seconds
-        self._watchdog_task: Optional[asyncio.Task] = None
+        self._watchdog_task: asyncio.Task | None = None
         self._cache_policy = CachePolicy()
 
     @property
@@ -413,7 +434,7 @@ class GoogleNestSubscriber:
         try:
             self._subscriber_future = (
                 await self._subscriber_factory.async_new_subscriber(
-                    creds, self._subscriber_id, self._loop, self._async_message_callback
+                    creds, self._subscriber_id, self._loop, self._async_message_callback_with_timeout
                 )
             )
         except NotFound as err:
@@ -507,12 +528,25 @@ class GoogleNestSubscriber:
                 self._healthy = False
             _LOGGER.debug("Subscriber disconnected, will restart: %s: %s", type(ex), ex)
 
+    async def _async_message_callback_with_timeout(
+        self, message: pubsub_v1.subscriber.message.Message
+    ) -> None:
+        """Handle a received message."""
+        try:
+            async with asyncio.timeout(MESSAGE_ACK_TIMEOUT_SECONDS):
+                print("Async message callback with timeout, ", MESSAGE_ACK_TIMEOUT_SECONDS)
+                await self._async_message_callback(message)
+                print("Done")
+        except TimeoutError as err:
+            DIAGNOSTICS.increment("message_ack_timeout")
+            raise TimeoutError("Message ack timeout processing message") from err
+
     async def _async_message_callback(
         self, message: pubsub_v1.subscriber.message.Message
     ) -> None:
         """Handle a received message."""
         payload = json.loads(bytes.decode(message.data))
-        event = EventMessage(payload, self._auth)
+        event = EventMessage.create_event(payload, self._auth)
         recv = time.time()
         latency_ms = int((recv - event.timestamp.timestamp()) * 1000)
         DIAGNOSTICS.elapsed("message_received", latency_ms)
@@ -525,8 +559,48 @@ class GoogleNestSubscriber:
             and self._device_manager_task.done()
             and not self._device_manager_task.exception()
         ):
-            await self._device_manager_task.result().async_handle_event(event)
+            device_manager = self._device_manager_task.result()
+            if _is_invalid_thermostat_trait_update(event):
+                _LOGGER.debug(
+                    "Ignoring event with invalid update traits; Refreshing devices: %s",
+                    event.resource_update_traits,
+                )
+                await _hack_refresh_devices(self._api, device_manager)
+            else:
+                await device_manager.async_handle_event(event)
             message.ack()
 
         ack_latency_ms = int((time.time() - recv) * 1000)
         DIAGNOSTICS.elapsed("message_acked", ack_latency_ms)
+
+
+def _is_invalid_thermostat_trait_update(event: EventMessage) -> bool:
+    """Return true if this is an invalid thermostat trait update."""
+    if (
+        event.resource_update_traits is not None
+        and (
+            thermostat_mode := event.resource_update_traits.get(
+                "sdm.devices.traits.ThermostatMode"
+            )
+        )
+        and (available_modes := thermostat_mode.get("availableModes")) is not None
+        and available_modes == ["OFF"]
+    ):
+        return True
+    return False
+
+
+async def _hack_refresh_devices(
+    api: GoogleNestAPI, device_manager: DeviceManager
+) -> None:
+    """Update the device manager with refreshed devices from the API."""
+    DIAGNOSTICS.increment("invalid-thermostat-update")
+    try:
+        devices = await api.async_get_devices()
+    except ApiException:
+        DIAGNOSTICS.increment("invalid-thermostat-update-refresh-failure")
+        _LOGGER.debug("Failed to refresh devices after invalid message")
+    else:
+        DIAGNOSTICS.increment("invalid-thermostat-update-refresh-success")
+        for device in devices:
+            device_manager.add_device(device)
